-- =====================================================
--
--   EPP interface
--
-- =====================================================
--
-- This is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2, or (at your option)
-- any later version.
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this package; see the file COPYING.  If not, write to
-- the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-- Boston, MA 02111-1307, USA.

-- $Id: epp.vhd,v 1.7 2006/08/25 12:05:06 jsmrz Exp $
--
-- $Log: epp.vhd,v $
-- Revision 1.7  2006/08/25 12:05:06  jsmrz
-- Added trigger edge detectors,
-- optimized timing.
--
-- Revision 1.6  2005/07/19 10:06:57  jsmrz
-- Signal postrig_on renamed to pretrig_off,
-- corrected extrig_val and extrig_en assignment,
-- added signal inv_trig.
--
-- Revision 1.5.2.3  2005/04/21 13:09:07  jsmrz
-- added license info
--
-- Revision 1.5.2.2  2005/04/21 12:54:25  jsmrz
-- english commentary
--
-- Revision 1.5.2.1  2005/04/20 14:47:19  jsmrz
-- added switch pretrig off
--
-- Revision 1.5  2005/04/20 12:11:57  jsmrz
-- EPP access synchronised to miniLA clock
-- added auto-incrementation of bytesel and address counter
-- added STOP functionality, rising/falling edge support
-- posttrigger register changed to pretrigger reg. and resized to 4bits.
-- unused registers removed
-- reading of data from address 12-15 removed
-- outputs are in Hi-Z when reading unimplemented registers
--
-- Revision 1.4  2005/02/07 16:34:44  jsmrz
-- EPP interface
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity epp is
   generic
   (
      CPLD_FW_VER	: std_logic_vector(3 downto 0);		-- CPLD firmware version
      HW_VER		: std_logic_vector(2 downto 0)		-- HW version
   );
   port
   (
      -- LPT interface
      i			: in std_logic_vector(7 downto 0);	-- EPP input data bus
      o			: out std_logic_vector(7 downto 0);	-- EPP output data bus
      ALE_l		: in std_logic;				-- EPP address strobe
      RD_l		: in std_logic;				-- EPP data strobe
      WR_l		: in std_logic;				-- EPP read/write
      d_oe		: out std_logic;			-- EPP data output enable

      -- input signals
      clk		: in std_logic;				-- clock input
      sd		: in std_logic_vector(31 downto 0);	-- SRAM data bus
      done		: in std_logic;				-- sampling completed
      trig		: in std_logic;				-- trigger detected
      sample_cnt_top	: in std_logic;				-- all data stored(read)

      -- output signals
      run		: out std_logic;			-- start sampling
      stop		: out std_logic;			-- stop sampling
      clr		: out std_logic;			-- miniLA reset
      clk_rd		: out std_logic;			-- memory read clock
      tcnt_reg		: out std_logic_vector(3 downto 0); 	-- trigger event counter
      tlen_reg		: out std_logic_vector(3 downto 0); 	-- trigger length counter
      tsel_reg		: out std_logic_vector(4 downto 0); 	-- timebase selector
      clk_falling_edge	: out std_logic;			-- falling edge enable
      psize_reg		: out std_logic_vector(3 downto 0); 	-- pre/posttrigger length
      pretrig_off	: out std_logic;
      mask_reg		: out std_logic_vector(15 downto 0);	-- trigger mask
      edge_reg		: out std_logic_vector(15 downto 0); 	-- edge/level sensitivity
      extrig_en 	: out std_logic;			-- external trigger enable
      extrig_val	: out std_logic;			-- external trigger active level
      inv_trig		: out std_logic;			-- invert internal trigger
      value_reg		: out std_logic_vector(15 downto 0) 	-- trigger value
    );
end entity epp;

architecture rtl of epp is

   signal adr		: std_logic_vector(3 downto 0);		-- EPP address
   signal run_int	: std_logic;
   signal stop_int	: std_logic;
   signal clr_int	: std_logic;
   signal clk_rd_int	: std_logic;
   signal auto_inc	: std_logic;
   signal bytesel	: std_logic_vector(1 downto 0);		-- output data bus byte selector
   signal ALE_l_i	: std_logic;
   signal RD_l_i	: std_logic;
   signal WR_l_i	: std_logic;
   signal a_wr_i	: std_logic;				-- address write strobe
   signal d_wr_i	: std_logic;				-- data write strobe
   signal d_rd_i	: std_logic;				-- data read strobe
   signal d_rd_i_d	: std_logic;
  

begin
   
   -- registering for miniLA time domain
   process(clk)
   begin
      if clk'event and clk='1' then
         ALE_l_i <= ALE_l;
	 RD_l_i <= RD_l;
	 WR_l_i <= WR_l;
      end if;
   end process;
    
   -- WAIT
   --process(clk)
   --begin
      --if clk'event and clk='1' then
        -- if (a_stb_l_i = '0') or (d_stb_l_i = '0') then
	   -- wait_o <= '1';
	-- else
   --	    wait_o <= '0';
	-- end if;
   --   end if;
   --end process;
   
   -- address write strobe
   a_wr_i <= '1' when (ALE_l_i  = '1') and (WR_l_i = '1') and (RD_l_i = '1') else
   	     '0';

   -- EPP address register
   process(clk)
   begin
      if clk'event and clk='1' then
         if a_wr_i='1' then
            adr <= i(3 downto 0);
	 end if;
      end if;
   end process;
   
   -- data write strobe
   d_wr_i <= '1' when (ALE_l_i  = '0') and (WR_l_i = '0') and (RD_l_i = '1') else
   	     '0';
   
   -- data read strobe
   d_rd_i <= '1' when (ALE_l_i  = '0') and (WR_l_i = '1') and (RD_l_i = '0') else
             '0';
   
   -- d_rd_idelay
   process(clk)
   begin
      if clk'event and clk='1' then
         d_rd_i_d <= d_rd_i;
      end if;
   end process;
  
   -- control_reg (s)
   process(clk)
   begin
      if clk'event and clk='1' then
         if (adr = conv_std_logic_vector(0, 4)) then
	    if d_wr_i = '1' then				-- control register write
	       run_int <= i(7);
	       clr_int <= i(6);
	       clk_rd_int <= i(5);
	       auto_inc <= i(4);
	       stop_int <= i(3);
	       bytesel <= i(1 downto 0);
	    elsif (d_rd_i = '0') and (d_rd_i_d = '1') then	-- end of read of data register
	       run_int <= run_int;
	       clr_int <= clr_int;
	       auto_inc <= auto_inc;
	       stop_int <= stop_int;
	       bytesel <= bytesel + conv_std_logic_vector(1, 2);-- autoincrementation of bytesel
	       if (bytesel = "11") and (auto_inc = '1') then	-- memory counter clock generation
	          clk_rd_int <= '1';
	       else
	          clk_rd_int <= '0';
	       end if;
	    else
	       run_int <= run_int;
	       clr_int <= clr_int;
	       auto_inc <= auto_inc;
	       stop_int <= stop_int;
	       clk_rd_int <= '0';
	       bytesel <= bytesel;
	    end if;
	 end if;
      end if;
   end process;

   -- tcnt_reg (j)
   process(clk)
   begin
      if clk'event and clk='0' then
         if (d_wr_i = '1') and (adr = conv_std_logic_vector(1, 4)) then
	    tcnt_reg <= i(3 downto 0);
	 end if;
      end if;
   end process;

   -- tlen_reg (k)
   process(clk)
   begin
      if clk'event and clk='0' then
         if (d_wr_i = '1') and (adr = conv_std_logic_vector(2, 4)) then
	    tlen_reg <= i(3 downto 0);
	 end if;
      end if;
   end process;

   -- tsel_reg (b)
   process(clk)
   begin
      if clk'event and clk='0' then
         if (d_wr_i = '1') and (adr = conv_std_logic_vector(3, 4)) then
	    tsel_reg <= i(4 downto 0);
	    clk_falling_edge <= i(5);
	 end if;
      end if;
   end process;

   -- psize_reg (p)
   process(clk)
   begin
      if clk'event and clk='0' then
         if (d_wr_i = '1') and (adr = conv_std_logic_vector(4, 4)) then
	    psize_reg <= i(3 downto 0);
	    pretrig_off <= i(4);
	 end if;
      end if;
   end process;

   -- value_reg(7 downto 0) (x1)
   process(clk)
   begin
      if clk'event and clk='0' then
         if (d_wr_i = '1') and (adr = conv_std_logic_vector(5, 4)) then
	    value_reg(7 downto 0) <= i;
	 end if;
      end if;
   end process;

   -- value_reg(15 downto 8) (x2)
   process(clk)
   begin
      if clk'event and clk='0' then
         if (d_wr_i = '1') and (adr = conv_std_logic_vector(6, 4)) then
	    value_reg(15 downto 8) <= i;
	 end if;
      end if;
   end process;
   
   -- edge_reg(7 downto 0) (e1)
   process(clk)
   begin
      if clk'event and clk='0' then
         if (d_wr_i = '1') and (adr = conv_std_logic_vector(7, 4)) then
	    edge_reg(7 downto 0) <= i;
	 end if;
      end if;
   end process;
   
   -- edge_reg(15 downto 8) (e2)
   process(clk)
   begin
      if clk'event and clk='0' then
         if (d_wr_i = '1') and (adr = conv_std_logic_vector(8, 4)) then
	    edge_reg(15 downto 8) <= i;
	 end if;
      end if;
   end process;

   -- mask_reg(7 downto 0) (m1)
   process(clk)
   begin
      if clk'event and clk='0' then
         if (d_wr_i = '1') and (adr = conv_std_logic_vector(9, 4)) then
	    mask_reg(7 downto 0) <= i;
	 end if;
      end if;
   end process;

   -- mask_reg(15 downto 8) (m2)
   process(clk)
   begin
      if clk'event and clk='0' then
         if (d_wr_i = '1') and (adr = conv_std_logic_vector(10, 4)) then
	    mask_reg(15 downto 8) <= i;
	 end if;
      end if;
   end process;

   -- external trigger
   process(clk)
   begin
      if clk'event and clk='0' then
         if (d_wr_i = '1') and (adr = conv_std_logic_vector(13, 4)) then
	    extrig_val <= i(0);
	    extrig_en <= i(1);
	    inv_trig <= i(7);
	 end if;
      end if;
   end process;

  
   -- output multiplexer
   process(d_rd_i, adr, bytesel, sd, run_int, clr_int, trig, sample_cnt_top,  done)
   begin
      if d_rd_i = '1' then
	 case adr is
	    when X"0" =>
	       d_oe <= '1';
	       case bytesel is
	          when "00" =>
		     o <= sd(7 downto 0);
		  when "01" =>
		     o <= sd(15 downto 8);
		  when "10" => 
		     o <= sd(23 downto 16);
		  when others =>
		     o <= sd(31 downto 24);
	       end case;
	    when X"1" =>
	       d_oe <= '1';
	       o <= done & HW_VER & CPLD_FW_VER;
	    when X"3" =>
	       d_oe <= '1';
	       o <= done & run_int & clr_int & trig & sample_cnt_top &"000";
	    when others =>
	       d_oe <= '0';
	       o <= (others => '1');
	 end case;
      else
         d_oe <= '0';
         o <= (others => '1');
      end if;
   end process;
   
   -- assign internal signals to outputs
   run <= run_int;
   clr <= clr_int;
   clk_rd <= clk_rd_int;
   stop <= stop_int;

end rtl;
