-- =====================================================
--
--   miniLA
--
-- =====================================================
--
-- This is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2, or (at your option)
-- any later version.
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this package; see the file COPYING.  If not, write to
-- the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-- Boston, MA 02111-1307, USA.


--


-- $Id: minila.vhd,v 1.7 2006/08/25 12:05:06 jsmrz Exp $
--
-- $Log: minila.vhd,v $
-- Revision 1.7  2006/08/25 12:05:06  jsmrz
-- Added trigger edge detectors,
-- optimized timing.
--
-- Revision 1.6  2005/07/19 10:11:17  jsmrz
-- Input clk_100 renamed to clk_in,
-- signal postrig_on renamed to pretrig_off,
-- added signal inv_trig,
-- assignments to signal st in the state machine inverted,
-- sample_cnt_wm removed from condition at state st_prefill.
--
-- Revision 1.5.2.3  2005/04/21 13:10:42  jsmrz
-- added global buffer for internal clock
-- added license info
--
-- Revision 1.5.2.2  2005/04/21 12:51:15  jsmrz
-- pretrigger condition fixed
--
-- Revision 1.5.2.1  2005/04/20 14:51:01  jsmrz
-- added switch pretrig off
--
-- Revision 1.5  2005/04/20 12:15:34  jsmrz
-- sampling can be stopped
-- added pretrigger
-- data output for SRAM registered
-- generic TRIGGER_WIDTH removed
-- FW version set to 1.5
-- changes of signal names
--
-- Revision 1.4  2005/02/07 16:37:05  jsmrz
-- Initial revision
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.all;

entity miniLA is
   generic
   (
      CPLD_FW_VER	: std_logic_vector(3 downto 0) := "0111";	-- CPLD firmware version
      HW_VER		: std_logic_vector(2 downto 0) := "001"		-- HW version
   );
   port
   (
      -- EPP interface
      dd		: inout std_logic_vector(7 downto 0);
      dale_l		: in std_logic;
      drd_l		: in std_logic;
      dwr_l		: in std_logic;
      
      -- SRAM interface
      sram_d		: inout std_logic_vector(31 downto 0);
      sram_adr		: out std_logic_vector(16 downto 0);
      sram_ce_l		: out std_logic;
      sram_oe_l		: out std_logic;
      sram_we_l		: out std_logic;
      sram_adsc_l	: out std_logic;
      sram_clk		: out std_logic;

      -- channel input
      channel		: in std_logic_vector(31 downto 0);

      -- clock, trigger
      clk_in		: in std_logic;				-- clock input
      eclk		: in std_logic;				-- external clock
      clk_o		: inout std_logic;			-- clock output
      
      etrg		: in std_logic;				-- external trigger input
      trig_o		: out std_logic;			-- trigger output

      -- LED diodes
      st		: out std_logic_vector(3 downto 1);

      -- other signals
      clk_ob		: inout std_logic			-- internal clock (used for routing on global line)
   );
end entity miniLA;

architecture rtl of miniLA is

   signal dd_out	: std_logic_vector(7 downto 0);		-- EPP data out
   signal d_oe		: std_logic;				-- EPP data output enable
   
   signal run		: std_logic;				-- start sampling
   signal stop		: std_logic;				-- stop sampling
   signal done		: std_logic;				-- sampling completed
   signal clr		: std_logic;				-- miniLA reset
   
   signal clk_in_gb	: std_logic;				-- input clock (global line)
   signal clk_int_ob	: std_logic;				-- internal clock (standard line)
   signal clk_int_gb	: std_logic;				-- internal clock (global line)
   signal clk_rd	: std_logic;				-- SRAM clock (from EPP)

   signal clk_out_ob	: std_logic;				-- output clock (standard line)
   signal clk_out_gb	: std_logic;				-- output clock (global line)
   
   signal tcnt_reg	: std_logic_vector(3 downto 0); 	-- trigger event cnt.
   signal tlen_reg	: std_logic_vector(3 downto 0); 	-- trigger length cnt.
   signal tsel_reg	: std_logic_vector(4 downto 0); 	-- timebase selector
   signal clk_falling_edge: std_logic;				-- falling edge enable
   signal psize_reg	: std_logic_vector(3 downto 0); 	-- pre/postrigger length selector
   signal pretrig_off	: std_logic;				-- pretrigger disabled
   signal extrig_en	: std_logic;				-- external trigger enable
   signal extrig_val	: std_logic;				-- external trigger active level
   signal inv_trig	: std_logic;				-- invert trigger condition
   signal mask_reg	: std_logic_vector(15 downto 0);	-- trigger mask
   signal edge_reg	: std_logic_vector(15 downto 0); 	-- trigger edge/level
   signal value_reg	: std_logic_vector(15 downto 0); 	-- trigger level

   signal trig_on	: std_logic;				-- trigger on
   signal trig		: std_logic;				-- trigger hit

   signal sram_write	: std_logic;				-- SRAM write enable
   
   signal channel_reg	: std_logic_vector(31 downto 0);	-- channel data (delayed)
   signal adr_cnt_en	: std_logic;				-- address counter enable
   signal sample_cnt_en	: std_logic;				-- sample counter enable
   signal sample_cnt_wm : std_logic;				-- sample wm hit
   signal sample_cnt_top: std_logic;				-- sample top hit
   
   type t_state		is (st_idle, st_prefill, st_trig, st_fill, st_read, st_done);
   signal state		: t_state;

   -- output buffer
   component OBUF
      port
      ( O	: out std_ulogic;
        I	: in std_ulogic);
   end component;
    
   -- input global buffer
   component BUFG
      port
      (	 O	: out std_ulogic;
	 I	: in std_ulogic);
   end component;

begin
    
    -- route input clock on global line
   clk_in_bufg_I: BUFG
      port map
      (
         O => clk_in_gb,
	 I => clk_in
      );
   
   -- Timebase
   timebase_I: entity timebase
      port map
      (
         clr			=> clr,			-- in
	 clk_in			=> clk_in_gb,		-- in
	 clk_ext		=> eclk,		-- in
	 clk_rd			=> clk_rd,		-- in
	 clk_sel		=> tsel_reg,		-- in
	 clk_falling_edge	=> clk_falling_edge,	-- in
	 done			=> done,		-- in
	 clk_int_out		=> clk_int_ob,		-- out
	 clk_out_gb		=> clk_out_gb,		-- in
	 clk_out_e		=> clk_out_ob		-- out
      );
	 
  
   -- SRAM clock output
   sram_clk <= clk_int_ob;
   
   -- take internal clock to pad
   clk_int_ob_I: OBUF
      port map
      (
         O => clk_ob,
	 I => clk_int_ob
      );
 
    -- route internal clock on global line
    clk_int_bufg_I: BUFG
      port map
      (
         O => clk_int_gb,
	 I => clk_ob
      );
   
   -- take output clock to pad
   clk_out_ob_I: OBUF
      port map
      (
         O => clk_o,
	 I => clk_out_ob
      );
 
    -- route output clock on global line
    clk_out_bufg_I: BUFG
      port map
      (
         O => clk_out_gb,
	 I => clk_o
      );



   -- EPP interface
   epp_I: entity epp
      generic map
      (
         CPLD_FW_VER	=> CPLD_FW_VER,
	 HW_VER		=> HW_VER
      )
      port map
      (
	 -- LPT interface
	 i		=> dd,				-- in
	 o		=> dd_out,			-- out
	 ALE_l	=> dale_l,			-- in
	 RD_l	=> drd_l,			-- in
	 WR_l	=> dwr_l,			-- in
	 d_oe		=> d_oe,

	 -- input signals
	 clk		=> clk_in_gb,			-- in
	 sd		=> sram_d,			-- in
	 done		=> done,			-- in
	 trig		=> trig,			-- in
	 sample_cnt_top	=> sample_cnt_top,		-- in

	 -- output signals
	 run		=> run,				-- out
	 stop		=> stop,			-- out
	 clr		=> clr,				-- out
	 clk_rd		=> clk_rd,			-- out
	 tcnt_reg	=> tcnt_reg,			-- out
	 tlen_reg	=> tlen_reg,			-- out
	 tsel_reg	=> tsel_reg,			-- out
	 clk_falling_edge => clk_falling_edge,		-- out
	 psize_reg	=> psize_reg,			-- out
	 pretrig_off	=> pretrig_off,			-- out
	 extrig_en	=> extrig_en,			-- out
	 extrig_val	=> extrig_val,			-- out
	 inv_trig	=> inv_trig,			-- out

	 mask_reg	=> mask_reg,			-- out
	 edge_reg	=> edge_reg,			-- out
	 value_reg	=> value_reg			-- out
      );

   -- output data tristate buffer
   dd <= dd_out when d_oe = '1' else
         (others => 'Z');

   
   trigcond_I: entity trigcond
      port map
      (
	 clr		=> clr,				-- in
	 clk		=> clk_int_gb,			-- in
	 
	 extrig		=> etrg,			-- in
	 extrig_en	=> extrig_en,			-- in
	 extrig_val	=> extrig_val,			-- in
	 inv_trig	=> inv_trig,			-- in

         channel	=> channel(15 downto 0),	-- in
	 mask_reg	=> mask_reg,			-- in
	 edge_reg	=> edge_reg,			-- in
	 value_reg	=> value_reg,			-- in
	 tlen_reg	=> tlen_reg,			-- in
	 tcnt_reg	=> tcnt_reg,			-- in
	 trig_on	=> trig_on,			-- in
	 trig		=> trig				-- out
      );

   -- trigger output
   trig_o <= trig;

   sram_I: entity sram
      port map
      (
	 clr		=> clr,				-- in
	 clk		=> clk_int_gb,			-- in
	 
         ce_l		=> sram_ce_l,			-- out
	 oe_l		=> sram_oe_l,			-- out
	 we_l		=> sram_we_l,			-- out
	 adsc_l		=> sram_adsc_l,			-- out
	 sram_write	=> sram_write,			-- in
	 adr_cnt_en	=> adr_cnt_en,			-- in
	 adr_cnt	=> sram_adr,			-- out
	 psize_reg	=> psize_reg,			-- in
	 sample_cnt_en	=> sample_cnt_en,		-- in
	 sample_cnt_wm	=> sample_cnt_wm,		-- out
	 sample_cnt_top	=> sample_cnt_top		-- out
      );

  
   -- channel data register
   process(clk_int_gb)
   begin
      if clk_int_gb'event and clk_int_gb = '1' then
	 channel_reg <= channel;
      end if;
   end process;
   
   -- SRAM data bus output enable
   sram_d <= channel_reg when sram_write = '1' else
             (others => 'Z');
   
   -- main state-machine
   process(clr, clk_int_gb)
   begin
      if clr = '1' then
         st <= "111";
	 trig_on <= '0';
	 sram_write <= '1';
	 adr_cnt_en <= '0';
	 sample_cnt_en <= '0';
	 done <= '0';
         state <= st_idle;
	 
      elsif clk_int_gb'event and clk_int_gb = '1' then
	 case state is
	    -- idle
	    when st_idle => 
	       st <= not("000");
	       trig_on <= '0';
	       sram_write <= '1';
	       adr_cnt_en <= '0';
	       sample_cnt_en <= '0';
	       done <= '0';
	       if run = '1' then
		  if (pretrig_off = '1') then		-- miniLA started, zero pretrigger
		     state <= st_trig;
		  else					-- miniLA started, non-zero pretrigger
		     state <= st_prefill;
		  end if;
	       else
		  state <= st_idle;
	       end if;
	       
	    -- memory prefill (pretrigger)
	    when st_prefill => 
	       st <= not("001");
	       trig_on <= '0';
	       sram_write <= '1';
	       adr_cnt_en <= '1';
	       sample_cnt_en <= '1';
	       done <= '0';
	       if stop = '1' then			-- sampling stopped
		  state <= st_read;
	       elsif (sample_cnt_wm = '1') then 	-- pretrigger filled
		  state <= st_trig;
	       else
		  state <= st_prefill;
	       end if;

	    -- waiting for trigger
	    when st_trig => 
	       st <= not("010");
	       trig_on <= '1';
	       sram_write <= '1';
	       adr_cnt_en <= '1';
	       sample_cnt_en <= '0';
	       done <= '0';
	       if stop = '1' then			-- miniLA stopped
		  state <= st_read;
	       elsif trig = '1' then			-- trigger detected
		  state <= st_fill;
	       else
		  state <= st_trig;
	       end if;

	    -- posttrigger
	    when st_fill => 
	       st <= not("011");
	       trig_on <= '0';
	       sram_write <= '1';
	       adr_cnt_en <= '1';
	       sample_cnt_en <= '1';
	       done <= '0';
	       if stop = '1' then			-- miniLA stopped
		  state <= st_read;
	       elsif (sample_cnt_top = '1') or ((pretrig_off = '1') and (sample_cnt_wm = '1')) then-- all data stored
		  state <= st_read;
	       else	
		  state <= st_fill;
	       end if;
	       
	    -- all data stored, switch for memory read
	    when st_read => 
	       st <= not("100");
	       trig_on <= '0';
	       sram_write <= '0';
	       adr_cnt_en <= '1';
	       sample_cnt_en <= '1';
	       done <= '0';
	       state <= st_done;
	       
	    -- transfer data via LPT
	    when st_done => 
	       st <= not("111");
	       trig_on <= '0';
	       sram_write <= '0';
	       adr_cnt_en <= '1';
	       sample_cnt_en <= '1';
	       done <= '1';
	       state <= st_done;
	 end case;
      end if;
   end process;


end rtl;

			   

