-- =====================================================
--
--   Timebase
--
-- =====================================================
--
-- This is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2, or (at your option)
-- any later version.
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this package; see the file COPYING.  If not, write to
-- the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-- Boston, MA 02111-1307, USA.


-- $Id: timebase.vhd,v 1.7 2006/08/25 12:05:06 jsmrz Exp $
--
-- $Log: timebase.vhd,v $
-- Revision 1.7  2006/08/25 12:05:06  jsmrz
-- Added trigger edge detectors,
-- optimized timing.
--
-- Revision 1.6  2005/07/19 10:20:18  jsmrz
-- no changes
--
-- Revision 1.5.2.3  2005/04/21 13:13:38  jsmrz
-- added global buffer for internal clock
-- added license info
--
-- Revision 1.5.2.2  2005/04/21 12:56:13  jsmrz
-- english commentary
--
-- Revision 1.5.2.1  2005/04/20 14:55:02  jsmrz
-- no changes
--
-- Revision 1.5  2005/04/20 12:19:28  jsmrz
-- timebase for 10Hz, 20Hz and 50Hz removed
-- polarity of output clock selectable
-- glitch-free switching to read clock
--
-- Revision 1.4  2005/02/07 16:40:45  jsmrz
-- Initial revision
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity timebase is
   port
   (
      clr		: in std_logic;			-- miniLA reset
      clk_in		: in std_logic;			-- main clock input
      clk_ext		: in std_logic;			-- external clock input
      clk_rd		: in std_logic;			-- memory read clock (from EPP)
      clk_sel		: in std_logic_vector(4 downto 0);-- timebase selector
      clk_falling_edge	: in std_logic;			-- falling edge enable
      done		: in std_logic;			-- sampling completed
      
      clk_int_out	: out std_logic;		-- miniLA internal clock
      clk_out_gb	: in std_logic;			-- miniLA output clock (brought back on global line)
      clk_out_e		: out std_logic			-- clock output (to pin)
   );
end entity timebase;

architecture rtl of timebase is
   
   signal clk_out_int	: std_logic;			-- internal/output clock output
   signal clk_cnt	: std_logic_vector(18 downto 0);-- timebase counter
   signal clk_2		: std_logic;			-- divider by 2
   signal sw_read_clk	: std_logic;			-- clock switch to EPP
   signal sw_read_clk_r	: std_logic;			-- clock switch to EPP
   signal sw_read_clk_f	: std_logic;			-- clock switch to EPP
   
begin

   -- clock divider
   process(clr, clk_in)
   begin
      if clr = '1' then
         clk_cnt <= (others => '0');
         clk_2 <= '0';
      elsif clk_in'event and clk_in = '1' then
         if clk_cnt = conv_std_logic_vector(0, 19) then
	    case clk_sel is
	       when "00010" => clk_cnt <= conv_std_logic_vector(       2-1, 19);	--  2 -> 20 MHz
	       when "00011" => clk_cnt <= conv_std_logic_vector(       4-1, 19);	--  3 -> 10 MHz
	       when "00100" => clk_cnt <= conv_std_logic_vector(       8-1, 19);	--  4 -> 5 MHz
	       when "00101" => clk_cnt <= conv_std_logic_vector(      20-1, 19);	--  5 -> 2 MHz
	       when "00110" => clk_cnt <= conv_std_logic_vector(      40-1, 19);	--  6 -> 1 MHz
	       when "00111" => clk_cnt <= conv_std_logic_vector(      80-1, 19);	--  7 -> 500 KHz
	       when "01000" => clk_cnt <= conv_std_logic_vector(     200-1, 19);	--  8 -> 200 KHz
	       when "01001" => clk_cnt <= conv_std_logic_vector(     400-1, 19);	--  9 -> 100 KHz
	       when "01010" => clk_cnt <= conv_std_logic_vector(     800-1, 19);	-- 10 -> 50 KHz
	       when "01011" => clk_cnt <= conv_std_logic_vector(    2000-1, 19);	-- 11 -> 20 KHz
	       when "01100" => clk_cnt <= conv_std_logic_vector(    4000-1, 19);	-- 12 -> 10 KHz
	       when "01101" => clk_cnt <= conv_std_logic_vector(    8000-1, 19);	-- 13 -> 5 KHz
	       when "01110" => clk_cnt <= conv_std_logic_vector(   20000-1, 19);	-- 14 -> 2 KHz
	       when "01111" => clk_cnt <= conv_std_logic_vector(   40000-1, 19);	-- 15 -> 1 KHz
	       when "10000" => clk_cnt <= conv_std_logic_vector(   80000-1, 19);	-- 16 -> 500 Hz
	       when "10001" => clk_cnt <= conv_std_logic_vector(  200000-1, 19);	-- 17 -> 200 Hz
	       when "10010" => clk_cnt <= conv_std_logic_vector(  400000-1, 19);	-- 18 -> 100 Hz
	       when others =>  clk_cnt <= conv_std_logic_vector(       1-1, 19);	--    -> 40 MHz
	    end case;
	    clk_2 <= not(clk_2);
	 else
	    clk_cnt <= clk_cnt - conv_std_logic_vector(1, 19);
	    clk_2 <= clk_2;
	 end if;
      end if;
   end process;
   
   -- output mux
   process(clk_sel, clk_in, clk_cnt, clk_2, clk_ext, clk_rd)
   begin
      case clk_sel is
	 when "00000" => -- 0
	    clk_out_int <= clk_in;
	 when "11110" => -- 30
	    clk_out_int <= clk_ext;
	 when "11111" => -- 31
	    clk_out_int <= clk_rd;
	 when others =>
	    clk_out_int <= clk_2;
      end case;
   end process;



   -- output clock (to pin)
   clk_out_e <= clk_out_int;
 
   -- switch to memory read clocks
   -- (in order to avoid glitches on clock signal, clock signal is switched in inactive level)
   
   -- rising edge
   process(clr, clk_out_gb)
   begin
      if clr = '1' then
	 sw_read_clk_r <= '0';
      elsif clk_out_gb'event and clk_out_gb = '1' then
         if done = '1' then
	    sw_read_clk_r <= '1';
	 else
	    sw_read_clk_r <= '0';
	 end if;
      end if;
   end process;

   -- falling edge
   process(clr, clk_out_gb)
   begin
      if clr = '1' then
	 sw_read_clk_f <= '0';
      elsif clk_out_gb'event and clk_out_gb = '0' then
         if done = '1' then
	    sw_read_clk_f <= '1';
	 else
	    sw_read_clk_f <= '0';
	 end if;
      end if;
   end process;

   sw_read_clk <= sw_read_clk_r or sw_read_clk_f;   
  
   clk_int_out <= (clk_out_int xor clk_falling_edge) when sw_read_clk = '0' else
                   clk_rd;
 

end rtl;

