-- =====================================================
--
--   Trigger
--
-- =====================================================
--
-- This is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2, or (at your option)
-- any later version.
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this package; see the file COPYING.  If not, write to
-- the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-- Boston, MA 02111-1307, USA.


-- $Id: trigcond.vhd,v 1.7 2006/08/25 12:05:06 jsmrz Exp $
--
-- $Log: trigcond.vhd,v $
-- Revision 1.7  2006/08/25 12:05:06  jsmrz
-- Added trigger edge detectors,
-- optimized timing.
--
-- Revision 1.6  2005/07/19 10:23:19  jsmrz
-- External and internal trigger input registered.
--
-- Revision 1.5.2.3  2005/04/21 13:15:10  jsmrz
-- register added to external trigger
-- added license info
--
-- Revision 1.5.2.2  2005/04/21 12:57:01  jsmrz
-- english commentary
--
-- Revision 1.5.2.1  2005/04/20 15:35:34  jsmrz
-- no changes
--
-- Revision 1.5  2005/02/02 11:38:01  jsmrz
-- odstranen generic TRIGGER_WIDTH
--
-- Revision 1.4  2005/01/22 18:11:23  jsmrz
-- opravena poskozena predchozi revize
--
-- Revision 1.3  2005/01/22 15:57:22  jsmrz
-- pridan externi trigger
-- opraven reset v procesech
--
-- Revision 1.2  2004-12-02 15:19:54+01  jsmrz
-- opravena chyba pri kratkych pulzech
--
-- Revision 1.1  2004-12-02 12:15:53+01  jsmrz
-- Initial revision
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.minila_pkg.all;


entity trigcond is
   port
   (
      clr		: in std_logic;				-- miniLA reset
      clk		: in std_logic;				-- clock input

      extrig		: in std_logic;				-- external trigger input
      extrig_en		: in std_logic;				-- external trigger enable
      extrig_val	: in std_logic;				-- external trigger value
      inv_trig		: in std_logic;				-- invert trigger condition

      channel		: in std_logic_vector(15 downto 0);	-- channel input
      mask_reg		: in std_logic_vector(15 downto 0);	-- trigger mask
      edge_reg		: in std_logic_vector(15 downto 0);	-- trigger edge/level
      value_reg		: in std_logic_vector(15 downto 0);	-- trigger value
      tlen_reg		: in std_logic_vector(3 downto 0);	-- trigger length
      tcnt_reg		: in std_logic_vector(3 downto 0);	-- trigger event cnt.
      trig_on		: in std_logic;				-- trigger enable
      trig		: out std_logic				-- trigger output
   );
end entity trigcond;

architecture rtl of trigcond is

   signal extrig_int		: std_logic;
   
   signal int_vtrig_comp	: std_logic_vector(15 downto 0);
   signal int_vtrig		: std_logic;
   signal int_etrig_hit		: std_logic;
   signal int_etrig_hit_reg	: std_logic;
   signal int_trig_hit		: std_logic;
   signal int_trig_hit_reg	: std_logic;

   signal trig_hit		: std_logic;
   signal tlen_cnt		: std_logic_vector(3 downto 0);
   signal tlen_top		: std_logic;
   signal tlen_top_d		: std_logic;
   signal trig_cnt		: std_logic_vector(3 downto 0);
   signal trig_on_d		: std_logic;

begin
   
   -- comparation of expected and actual values
   int_vtrig_comp <= not(mask_reg) or (channel xnor value_reg);

   -- comparation of levels match
   int_vtrig <= wide_and(int_vtrig_comp);
   
   -- comparation of "inverted" levels match - for edge detection
   int_etrig_hit <= wide_nor(int_vtrig_comp(15 downto 0) and edge_reg(15 downto 0));

   -- value and edge comparation match
   int_trig_hit <= int_vtrig and int_etrig_hit_reg;

   -- register results of internal trigger
   process(clr, clk)
   begin
      if clr = '1' then
	 int_etrig_hit_reg <= '0';
         int_trig_hit_reg <= '0';
      elsif clk'event and clk = '1' then
	 int_etrig_hit_reg <= int_etrig_hit;
         int_trig_hit_reg <= int_trig_hit;
      end if;
   end process;

   -- register external trigger
   process(clr, clk)
   begin
      if clk'event and clk = '1' then
         extrig_int <= extrig;
      end if;
   end process;
   
   -- delay trig_on
   process(clr,clk)
   begin
      if clr = '1' then
         trig_on_d <= '0';
      elsif clk'event and clk = '1' then
         trig_on_d <= trig_on;
      end if;
   end process;
			 
   trig_hit <=    (not(extrig_en) and (int_trig_hit_reg xor inv_trig))	-- internal trigger
               or (    extrig_en  and (extrig_int xnor extrig_val));	-- external trigger
   
   -- trigger-hit length counter
   process(clr, clk)
   begin
      if clr = '1' then
         tlen_cnt <= (others => '0');
      elsif clk'event and clk = '1' then
	 if (trig_on_d = '0') or (trig_hit = '0') then 
            tlen_cnt <= conv_std_logic_vector(0, 4);
	 elsif tlen_top = '0' then
            tlen_cnt <= tlen_cnt + conv_std_logic_vector(1, 4);
	 else
	    tlen_cnt <= tlen_cnt;
	 end if;
      end if;
   end process;

   tlen_top <= '1' when tlen_cnt = tlen_reg else
               '0';

   -- t_len_top delay
   process(clr, clk)
   begin
      if clr = '1' then
	 tlen_top_d <= '0';
      elsif clk'event and clk = '1' then
	 tlen_top_d <= tlen_top;
      end if;
   end process;

   -- trigger-hit counter
   process(clr, clk)
   begin
      if clr = '1' then
         trig_cnt <= (others => '0');
      elsif clk'event and clk = '1' then
         if (tlen_top = '1') and (tlen_top_d = '0') then
            trig_cnt <= trig_cnt + conv_std_logic_vector(1, 4);
	 else
	    trig_cnt <= trig_cnt;
	 end if;
      end if;
   end process;

   trig <= '1' when trig_cnt = tcnt_reg else
           '0';

end rtl;
 
